import os
import time
import csv
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

# ChromeDriver 경로 설정
chrome_driver_path = "C:/tools/chromedriver/chromedriver.exe"
chrome_options = Options()
chrome_options.add_argument("--ignore-certificate-errors")

# Service 객체 생성
service = Service(executable_path=chrome_driver_path)

# Chrome 드라이버 실행
driver = webdriver.Chrome(service=service, options=chrome_options)

# 결과를 저장할 폴더 경로 설정
output_folder = "output"
if not os.path.exists(output_folder):
    os.makedirs(output_folder)

# 수집을 허용하는 제품 이름 목록 (CPU에서만 적용)
ALLOWED_PRODUCTS = ['AMD', '인텔']

# 부품별 페이지 수 제한 설정
PAGE_LIMIT = {
    'cpu': 10,
    'cooler': 10,
    'mainboard': 10,
    'memory': 10,
    'gpu': 10,
    'ssd': 10,
    'hdd': 10,
    'case': 10,
    'psu': 10,
    'software': 3
}

# 카테고리별 JavaScript 함수 호출 인자 설정
CATEGORY_JS_FUNCTION = {
    'cpu': "category(873,2);",
    'cooler': "category(887,2);",
    'mainboard': "category(875,2);",
    'memory': "category(874,2);",
    'gpu': "category(876,2);",
    'ssd': "category(32617,2);",
    'hdd': "category(877,2);",
    'case': "category(879,2);",
    'psu': "category(880,2);",
    'software': "category(901,2);"
}

# 다나와에서 제품 정보 수집하는 함수
from selenium.webdriver.support import expected_conditions as EC

def collect_product_info(category):
    print(f"{category} 카테고리 페이지로 이동 중...")
    driver.get("https://shop.danawa.com/virtualestimate/?controller=estimateMain&methods=index&marketPlaceSeq=16&logger_kw=dnw_top_esti")

    try:
        print(f"{category} 카테고리 선택 중 (JavaScript 실행)...")
        driver.execute_script(CATEGORY_JS_FUNCTION[category])
        time.sleep(2)  # JavaScript 실행 후 2초 대기

        # 페이지를 다시 한 번 로드하여 첫 페이지를 확실하게 로딩
        driver.execute_script(CATEGORY_JS_FUNCTION[category])
        time.sleep(2)  # 두 번째 JavaScript 실행 후 2초 대기

        # 페이지가 완전히 로드될 때까지 대기
        WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((By.CSS_SELECTOR, '.tbl_list tbody tr'))
        )
        print(f"{category} 카테고리 페이지 로딩 완료.")

    except Exception as e:
        print(f"{category} 카테고리 선택 실패: {e}")
        return

    # 데이터 수집 로직
    products_data = []  # 새로운 카테고리로 들어갈 때마다 데이터를 초기화
    current_page = 1

    max_pages = PAGE_LIMIT[category]

    while current_page <= max_pages:
        try:
            # 제품 목록이 로드될 때까지 대기
            product_rows = WebDriverWait(driver, 20).until(
                EC.presence_of_all_elements_located((By.CSS_SELECTOR, '.tbl_list tbody tr'))
            )

            for product in product_rows:
                try:
                    product_name = product.find_element(By.CSS_SELECTOR, 'td.title_price p.subject a').text
                    product_image = product.find_element(By.CSS_SELECTOR, 'td.goods_img img').get_attribute('src')

                    if product_image.startswith("//"):
                        product_image = "https:" + product_image

                    try:
                        product_price = product.find_element(By.CSS_SELECTOR, 'p.low_price .prod_price').text.replace(',', '').replace('원', '').strip()
                        
                        if not product_price:
                            continue

                        product_price = int(product_price)
                        rounded_price = round(product_price, -3)
                        final_price = round(rounded_price * 1.1, -3)

                    except:
                        continue

                    if category == 'cpu':
                        if any(keyword in product_name for keyword in ALLOWED_PRODUCTS):
                            products_data.append({
                                'name': product_name,
                                'price': final_price,
                                'image_url': product_image
                            })
                    else:
                        if not any(keyword in product_name for keyword in ALLOWED_PRODUCTS):
                            products_data.append({
                                'name': product_name,
                                'price': final_price,
                                'image_url': product_image
                            })

                except Exception as e:
                    print(f"제품 정보 추출 중 오류: {e}")

            save_to_csv(products_data, f"{category}.csv")

            current_page += 1
            try:
                next_page_button = driver.find_element(By.LINK_TEXT, str(current_page))
                next_page_button.click()
                time.sleep(5)

            except Exception as e:
                print(f"더 이상 페이지가 없습니다. 페이지 이동 불가: {e}")
                break

        except Exception as e:
            print(f"페이지 처리 중 오류 발생: {e}")
            break

    save_to_csv(products_data, f"{category}.csv")
    print(f"{category}의 모든 페이지에서 제품 정보가 수집되고 {category}.csv에 저장되었습니다.")


# 수집된 데이터를 CSV 파일로 저장하는 함수
def save_to_csv(products_data, filename):
    output_file = os.path.join(output_folder, filename)
    with open(output_file, mode='w', newline='', encoding='utf-8') as file:
        fieldnames = ['name', 'price', 'image_url']
        writer = csv.DictWriter(file, fieldnames=fieldnames)
        writer.writeheader()  # 헤더 작성
        for product in products_data:
            writer.writerow(product)

    print(f"수집된 데이터가 {output_file}에 저장되었습니다.")

# 메인 실행 함수
def main():
    categories = ['cpu', 'cooler', 'mainboard', 'memory', 'gpu', 'ssd', 'hdd', 'case', 'psu', 'software']
    
    for category in categories:
        print(f"{category} 정보 수집 시작...")
        collect_product_info(category)

if __name__ == "__main__":
    main()
