import csv
import re
import os

# 제품 데이터를 처리하고 부품별로 앞 3글자 기준으로 정렬 후 가격순으로 정렬하는 함수
def preprocess_product_data(directory="output", output_folder="sorted_parts"):
    # 부품별 파일 리스트
    part_files = [
        "cpu.csv", "cooler.csv", "gpu.csv", "mainboard.csv", "memory.csv",
        "ssd.csv", "hdd.csv", "case.csv", "psu.csv", "software.csv"
    ]

    # 출력 폴더 설정 (인수로 받은 출력 폴더를 사용)
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # 각 부품 파일에 대해 데이터를 처리
    for part_file in part_files:
        filepath = os.path.join(directory, part_file)
        all_products = []  # 모든 제품을 저장할 리스트
        seen_products = set()  # 중복된 제품 이름을 추적하기 위한 세트

        # 부품 이름 추출 (파일명에서 확장자를 제외한 부분)
        part_name = part_file.replace(".csv", "")

        # CSV 파일에서 데이터 읽기
        with open(filepath, mode='r', encoding='utf-8') as file:
            reader = csv.DictReader(file)
            for row in reader:
                product_name = row['name']
                price = int(float(row['price']))  # 가격을 float에서 정수로 변환
                image_url = row['image_url']

                # 중복 제품 확인 (제품 이름을 기준으로)
                if product_name not in seen_products:
                    # 제품 이름이 중복되지 않으면 추가
                    seen_products.add(product_name)

                    # 제품 이름 앞 3글자를 추출
                    prefix = product_name[:3]

                    # 제품 데이터를 리스트에 저장 (prefix는 정렬 용도로만 사용)
                    all_products.append({
                        'name': product_name,
                        'price': price,  # 가격을 정수형으로 저장
                        'image_url': image_url,
                        'prefix': prefix  # 정렬을 위한 용도로만 사용
                    })

        # 앞 3글자 기준으로 먼저 정렬하고, 그 안에서 가격 순으로 정렬
        all_products = sorted(all_products, key=lambda x: (x['prefix'], x['price']))

        # 정리된 데이터를 저장 (파일명 인수로 받은 값으로 사용)
        output_filepath = os.path.join(output_folder, f"sorted_{part_name}.csv")
        save_preprocessed_data(all_products, output_filepath)

        print(f"{part_name} 처리 완료: {output_filepath}에 저장되었습니다.")

# 전처리된 데이터를 CSV 파일로 저장하는 함수 (prefix 제외)
def save_preprocessed_data(data, filename):
    with open(filename, mode='w', newline='', encoding='utf-8') as file:
        fieldnames = ['name', 'price', 'image_url']
        writer = csv.DictWriter(file, fieldnames=fieldnames)
        writer.writeheader()

        for product in data:
            writer.writerow({
                'name': product['name'],
                'price': product['price'],  # 가격은 정수형으로 출력
                'image_url': product['image_url']
            })

# 메인 함수 실행
preprocess_product_data("output")
