import tkinter as tk
from tkinter import ttk
import pandas as pd
import os
import re
from datetime import datetime

# CSV 파일 불러오는 함수 (사용자 경로에 맞게 수정)
def load_csv(filename):
    return pd.read_csv(f'C:/Users/user/projects/st/sorted_parts/{filename}.csv')

# 각 부품 CSV 불러오기
cpu_data = load_csv('sorted_cpu')
gpu_data = load_csv('sorted_gpu')
mainboard_data = load_csv('sorted_mainboard')
memory_data = load_csv('sorted_memory')
ssd_data = load_csv('sorted_ssd')
hdd_data = load_csv('sorted_hdd')
psu_data = load_csv('sorted_psu')
case_data = load_csv('sorted_case')
cooler_data = load_csv('sorted_cooler')
software_data = load_csv('sorted_software')

# CPU 브랜드를 확인하는 함수 (한글과 영어 모두 인식)
def get_cpu_brand(cpu_name):
    if "intel" in cpu_name.lower() or "인텔" in cpu_name:
        return "Intel"
    elif "amd" in cpu_name.lower() or "에이엠디" in cpu_name or "AMD" in cpu_name:
        return "AMD"
    else:
        return "Unknown"

# 가격을 가져오는 함수 (NaN 값을 0으로 처리)
def get_price(component_name, data):
    if component_name:
        try:
            price = data[data['name'] == component_name]['price'].values[0]
            return 0 if pd.isna(price) else price  # NaN 값은 0으로 처리
        except IndexError:
            print(f"{component_name}의 가격을 찾을 수 없습니다.")
            return 0
    return 0

# 숫자만 추출하는 함수 (바이트 객체 처리 및 문자열 변환)
def extract_numbers(name):
    if isinstance(name, bytes):
        name = name.decode('utf-8')  # 바이트 객체를 문자열로 변환
    return ''.join(re.findall(r'\d+', str(name)))  # 정규식을 사용하여 숫자만 추출

# 가격을 실시간으로 계산하여 표시하는 함수
def update_total_price():
    try:
        total_price = 0  # 총 가격 초기화

        # 선택된 부품들의 가격을 가져옴
        total_price += get_price(drop_downs['CPU'].get(), cpu_data)
        total_price += get_price(drop_downs['쿨러/튜닝'].get(), cooler_data)
        total_price += get_price(drop_downs['메인보드'].get(), mainboard_data)
        total_price += get_price(drop_downs['메모리'].get(), memory_data)
        total_price += get_price(drop_downs['그래픽카드'].get(), gpu_data)
        total_price += get_price(drop_downs['SSD'].get(), ssd_data)
        total_price += get_price(drop_downs['HDD'].get(), hdd_data)
        total_price += get_price(drop_downs['케이스'].get(), case_data)
        total_price += get_price(drop_downs['파워'].get(), psu_data)
        total_price += get_price(drop_downs['소프트웨어'].get(), software_data)

        # 공임비를 더함
        labor_cost = int(drop_downs['공임'].get())
        total_price += labor_cost

        # 총합 가격을 화면에 표시
        total_label.config(text=f"총합 가격: {total_price:,}원")
    
    except Exception as e:
        print(f"가격 계산 중 오류 발생: {e}")

import os
import pandas as pd

# 견적 파일 저장 함수
def save_estimate(selected_components):
    try:
        total_price = 0  # 총합 가격

        # 선택된 부품들을 가져오기
        cpu_name = selected_components['CPU'].get()
        gpu_name = selected_components['그래픽카드'].get() if selected_components['그래픽카드'].get() else None

        # 부품별 가격 합산
        total_price += get_price(cpu_name, cpu_data)
        total_price += get_price(gpu_name, gpu_data) if gpu_name else 0
        total_price += get_price(selected_components['메인보드'].get(), mainboard_data)
        total_price += get_price(selected_components['메모리'].get(), memory_data)
        total_price += get_price(selected_components['SSD'].get(), ssd_data)
        total_price += get_price(selected_components['HDD'].get(), hdd_data)
        total_price += get_price(selected_components['파워'].get(), psu_data)
        total_price += get_price(selected_components['케이스'].get(), case_data)
        total_price += get_price(selected_components['쿨러/튜닝'].get(), cooler_data)
        total_price += get_price(selected_components['소프트웨어'].get(), software_data)

        # 공임비 추가
        labor_cost = int(selected_components['공임'].get())
        total_price += labor_cost

        # CPU와 GPU 이름에서 숫자만 추출
        cpu_numbers = extract_numbers(cpu_name)
        gpu_numbers = extract_numbers(gpu_name) if gpu_name else None

        # CPU 브랜드 확인
        cpu_brand = get_cpu_brand(cpu_name)

        # 파일명 설정 (GPU 선택 여부에 따라 파일명 다르게 처리)
        timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
        if gpu_numbers:
            new_filename = f"{cpu_brand}_{cpu_numbers}_{gpu_numbers}_{str(total_price)}_{timestamp}.csv"
        else:
            new_filename = f"{cpu_brand}_{cpu_numbers}_{str(total_price)}_{timestamp}.csv"
        
        save_path = f"C:/Users/user/projects/st/data/naver_good/{new_filename}"

        # 선택된 부품과 총합 가격을 저장
        estimate_data = {
            'CPU': cpu_name,
            '그래픽카드': gpu_name if gpu_name else "None",
            '메인보드': selected_components['메인보드'].get(),
            '메모리': selected_components['메모리'].get(),
            'SSD': selected_components['SSD'].get(),
            'HDD': selected_components['HDD'].get(),
            '파워': selected_components['파워'].get(),
            '케이스': selected_components['케이스'].get(),
            '쿨러/튜닝': selected_components['쿨러/튜닝'].get(),
            '소프트웨어': selected_components['소프트웨어'].get(),
            '공임': labor_cost,
            'Total Price': total_price
        }

        # 데이터프레임으로 변환 후 CSV 저장
        estimate_df = pd.DataFrame([estimate_data])
        estimate_df.to_csv(save_path, index=False)
        
        print(f"견적 저장 완료: {save_path}")
        
        # 저장된 파일을 불러오기
        loaded_df = pd.read_csv(save_path)

        # 폴더 생성 (new_estimates 폴더가 없으면 생성)
        new_folder = f"C:/Users/user/projects/st/data/naver_good/new_estimates/"
        os.makedirs(new_folder, exist_ok=True)

        # 세로로 정렬하여 저장할 파일명 설정 (원래 파일명 유지)
        new_save_path = os.path.join(new_folder, new_filename)

        # 파일을 세로로 변환하여 저장
        transposed_df = loaded_df.T
        transposed_df.to_csv(new_save_path, header=False)  # 세로로 변환, header 제거

        print(f"새로운 폴더에 세로로 정렬된 견적 저장 완료: {new_save_path}")
    
    except Exception as e:
        print(f"견적 저장 중 오류 발생: {e}")

import os
import pandas as pd
import re

# CPU 이름을 파싱하여 불필요한 부분 제거하고, 읽기 쉬운 형식으로 변환
def get_cpu_info(cpu_name):
    # 괄호 안의 내용 제거, "인텔 코어" 제거
    cpu_name = re.sub(r'\([^)]*\)', '', cpu_name).strip()  # 괄호 안의 내용 제거
    cpu_name = cpu_name.replace('인텔 코어', '').strip()  # '인텔 코어' 제거
    # 예: 'i5-12400F' -> 'i5-12세대 12400F'
    cpu_pattern = re.compile(r'(i[3579]|Ryzen [3579])[- ]?(\d{2})(\d{3,4})([A-Za-z]*)')
    match = cpu_pattern.search(cpu_name)
    
    if match:
        series = match.group(1)  # i5, i7, Ryzen 5 등
        generation = match.group(2)  # 세대 정보 추출 (예: 12세대)
        model = match.group(3) + match.group(4)  # 모델명 (예: 12400F)
        return f'{series}-{generation}세대_{model}'
    return re.sub(r'\s+', '_', cpu_name).strip('_')  # 공백을 _로 변환하고 양끝의 _ 제거

# GPU 이름을 파싱하여 모델명과 메모리 용량만 추출하고 불필요한 공백 제거
def get_gpu_info(gpu_name):
    # 괄호 안의 내용 및 불필요한 설명 제거
    gpu_name = re.sub(r'\([^)]*\)', '', gpu_name).strip()  # 괄호 안의 내용 제거
    # 브랜드나 추가 설명 제거 (예: 'MSI 지포스 RTX 4060 벤투스 2X 블랙 OC D6 8GB' -> 'RTX 4060 8GB')
    gpu_name = re.sub(r'(지포스|GeForce|MSI|벤투스|OC|블랙|D6|2X|X)', '', gpu_name).strip()  # 불필요한 단어 제거
    # 예: 'RTX 4060 8GB' 추출
    gpu_pattern = re.compile(r'(RTX \d{3,4}) ?(\d{1,2}G?B)')
    match = gpu_pattern.search(gpu_name)
    
    if match:
        model = match.group(1).replace(' ', '_')  # GPU 모델명 (예: 4060), 공백을 _로 변환
        memory = match.group(2).replace(' ', '_')  # 메모리 크기 (예: 8GB), 공백을 _로 변환
        return f'{model}_{memory}'  # 모델명과 메모리 용량을 _로 결합
    return re.sub(r'\s+', '_', gpu_name).strip('_')  # 공백을 _로 변환하고 양끝의 _ 제거

# 메모리 이름을 파싱하여 용량 정보만 추출
def get_memory_info(memory_name):
    # 괄호 안의 내용에서 용량 정보만 추출
    memory_pattern = re.compile(r'\((\d{1,2}GB)\)')  # 괄호 안에 있는 용량 정보만 추출
    match = memory_pattern.search(memory_name)
    
    if match:
        memory_size = match.group(1).replace('GB', 'G')  # 메모리 크기 (16GB -> 16G)
        return f'DDR4_{memory_size}'  # 속도 제거하고 용량만 표시
    return re.sub(r'\s+', '_', memory_name).strip('_')  # 공백을 _로 변환하고 양끝의 _ 제거

# 가격 갱신 함수
def update_price(estimate_filename):
    try:
        # 기존 견적 파일 읽기
        estimate_path = f'C:/Users/user/projects/st/data/naver_good/{estimate_filename}'
        estimate = pd.read_csv(estimate_path)

        # 선택된 부품명을 가져와 가격을 최신으로 갱신
        cpu_name = estimate['CPU'].values[0]
        gpu_name = estimate['그래픽카드'].values[0] if '그래픽카드' in estimate.columns else None
        memory_name = estimate['메모리'].values[0]
        mainboard_name = estimate['메인보드'].values[0]
        ssd_name = estimate['SSD'].values[0]
        hdd_name = estimate['HDD'].values[0]
        psu_name = estimate['파워'].values[0]
        case_name = estimate['케이스'].values[0]
        cooler_name = estimate['쿨러/튜닝'].values[0]
        software_name = estimate['소프트웨어'].values[0]
        labor_cost = estimate['공임'].values[0]

        # 최신 가격을 계산
        total_price = 0
        total_price += get_price(cpu_name, cpu_data)
        total_price += get_price(gpu_name, gpu_data) if gpu_name else 0
        total_price += get_price(mainboard_name, mainboard_data)
        total_price += get_price(memory_name, memory_data)
        total_price += get_price(ssd_name, ssd_data)
        total_price += get_price(hdd_name, hdd_data)
        total_price += get_price(psu_name, psu_data)
        total_price += get_price(case_name, case_data)
        total_price += get_price(cooler_name, cooler_data)
        total_price += get_price(software_name, software_data)
        total_price += labor_cost

        # 총합 가격 갱신
        estimate['Total Price'] = total_price

        # CPU, GPU, 메모리 정보를 파싱하여 읽기 좋은 형식으로 변경
        cpu_info = get_cpu_info(cpu_name)
        gpu_info = get_gpu_info(gpu_name) if gpu_name else 'None'
        memory_info = get_memory_info(memory_name)

        # 새로운 파일명 생성 (예시: i5-12세대_12400F_RTX_4060_8GB_DDR4_16G_1079000)
        new_filename = f"{cpu_info}_{gpu_info}_{memory_info}_{total_price}.csv"

        # 기존 파일명을 새로운 파일명으로 변경하기 전에, 기존 파일 삭제
        new_save_path = f'C:/Users/user/projects/st/data/naver_good/{new_filename}'
        if estimate_path != new_save_path and os.path.exists(estimate_path):
            os.remove(estimate_path)  # 기존 파일 삭제
            print(f"기존 파일 삭제: {estimate_path}")

        # 새로운 파일로 저장
        estimate.to_csv(new_save_path, index=False)  # 수정된 파일명으로 저장
        print(f"견적 파일 갱신 완료: {new_save_path}")

        # 세로로 정렬된 데이터도 갱신하여 덮어쓰기 (new_estimates 폴더)
        new_folder = f"C:/Users/user/projects/st/data/naver_good/new_estimates/"
        os.makedirs(new_folder, exist_ok=True)

        new_vertical_save_path = os.path.join(new_folder, new_filename)
        estimate_transposed = estimate.T

        # 기존 세로 파일이 존재하면 삭제
        old_vertical_path = os.path.join(new_folder, estimate_filename)
        if old_vertical_path != new_vertical_save_path and os.path.exists(old_vertical_path):
            os.remove(old_vertical_path)  # 기존 세로 파일 삭제
            print(f"기존 세로 파일 삭제: {old_vertical_path}")

        # 세로 파일 저장
        estimate_transposed.to_csv(new_vertical_save_path, header=False)
        print(f"세로로 정렬된 파일 갱신 완료: {new_vertical_save_path}")

    except Exception as e:
        print(f"갱신 중 오류 발생: {e}")


# 견적 파일 선택 및 갱신 실행
def refresh_all_estimates():
    estimate_files = [f for f in os.listdir('C:/Users/user/projects/st/data/naver_good') if f.endswith('.csv')]  # 견적 파일을 찾음
    for estimate_file in estimate_files:
        update_price(estimate_file)

# 드롭다운 검색 필터링
def filter_combobox(event, data, dropdown):
    value = event.widget.get()
    if value == '':
        dropdown['values'] = list(data['name'])  # 필터를 해제하고 전체 리스트를 다시 설정
    else:
        filtered_values = [item for item in list(data['name']) if value.lower() in item.lower()]
        dropdown['values'] = filtered_values  # 필터링된 리스트로 설정

    # 엔터 키를 눌렀을 때 드롭다운 자동으로 열리기
    if event.keysym == "Return":
        dropdown.event_generate("<Down>")  # 드롭다운을 열어주는 명령

# 드롭다운 메뉴 생성 함수
def create_dropdown(category, data):
    label = tk.Label(root, text=category)
    label.pack(padx=10, pady=5)  # 간격 조정
    
    if category == '공임':
        # 공임 드롭다운은 80,000원 또는 120,000원 선택 가능
        drop_down = ttk.Combobox(root, values=[80000, 120000], width=40)
        drop_down.set(80000)  # 기본값 설정
    else:
        # 부품 드롭다운 생성, 폭 조정
        drop_down = ttk.Combobox(root, values=list(data['name']), width=50)
    
    drop_down.pack(padx=10, pady=5)  # 간격 추가
    drop_downs[category] = drop_down

    # 드롭다운 필터링 이벤트 바인딩
    drop_down.bind("<KeyRelease>", lambda event: filter_combobox(event, data, drop_down))

    # 드롭다운 변경 시 가격 업데이트
    drop_down.bind("<<ComboboxSelected>>", lambda event: update_total_price())

# Tkinter GUI 구성
root = tk.Tk()
root.title("조립 컴퓨터 견적 갱신")

# 드롭다운 메뉴 프레임
categories = ['CPU', '쿨러/튜닝', '메인보드', '메모리', '그래픽카드', 'SSD', 'HDD', '케이스', '파워', '소프트웨어', '공임']
drop_downs = {}

# 각 카테고리별 드롭다운 생성
create_dropdown('CPU', cpu_data)
create_dropdown('쿨러/튜닝', cooler_data)
create_dropdown('메인보드', mainboard_data)
create_dropdown('메모리', memory_data)
create_dropdown('그래픽카드', gpu_data)
create_dropdown('SSD', ssd_data)
create_dropdown('HDD', hdd_data)
create_dropdown('케이스', case_data)
create_dropdown('파워', psu_data)
create_dropdown('소프트웨어', software_data)
create_dropdown('공임', None)  # 공임 드롭다운 생성

# 총합 가격 표시
total_label = tk.Label(root, text="총합 가격: 0원", width=40)
total_label.pack(pady=10)

# 저장 버튼
save_button = tk.Button(root, text="견적 저장", command=lambda: save_estimate(drop_downs))
save_button.pack(pady=10)

# 갱신 버튼
refresh_button = tk.Button(root, text="모든 견적 갱신", command=refresh_all_estimates)
refresh_button.pack(pady=10)

root.mainloop()
